@@ -92,7 +92,7 @@ class Bucket(object):
     def __str__(self):
         return "<Bucket items={}>".format(len(self.items))
 
-    def next(self):
+    def __next__(self):
         if self.__current < len(self.items):
             item = self.items[self.__current]
             self.__current += 1
@@ -101,6 +101,8 @@ class Bucket(object):
         self.__current = 0
         raise StopIteration
 
+    next = __next__
+
 
 class Collection(object):
 
@@ -138,7 +140,7 @@ class Collection(object):
     def __iter__(self):
         return self
 
-    def next(self):
+    def __next__(self):
         if self.__last is not None:
             self._set_current_status(not bool(self.__last.failed_report))
             self.__last = self._get_current_tests()
@@ -150,6 +152,8 @@ class Collection(object):
             raise StopIteration
         return self.__last
 
+    next = __next__
+
     def prepare(self, test_name):
         items = self._needed_tests(test_name)
         self._bts.insert(items)
@@ -307,14 +311,14 @@ class Sherlock(object):
         else:
             message = str(failed_report.longrepr)
         failed_report.longrepr = "\n{}\n\n{}".format(write_coupled_report(coupled), message)
-        node_reporter = _NodeReporter(target_item.nodeid, self.config._xml)
-        node_reporter.append_failure(failed_report)
-        node_reporter.finalize()
-        self.config._xml.node_reporters_ordered[:] = [node_reporter]
-        self.config._xml.stats["failure"] = 1
-        reports = self.reporter.getreports("failed")
-        last_report = reports[-1]
-        self.reporter.stats["failed"] = [last_report]
+        self.reporter.stats["failed"] = [failed_report]
+        xml = getattr(self.config, "_xml", None)
+        if xml:
+            node_reporter = _NodeReporter(target_item.nodeid, xml)
+            node_reporter.append_failure(failed_report)
+            node_reporter.finalize()
+            xml.node_reporters_ordered[:] = [node_reporter]
+            xml.stats["failure"] = 1
 
     @pytest.hookimpl(hookwrapper=True)
     def pytest_runtest_protocol(self, item, nextitem=None):
